ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula Ciutada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaCiutada (
  p_codiAleatori IN Ciutada.codiAleatori%TYPE,
  p_pais IN Pais.codiPais%TYPE,
  p_regio IN Regio.codiRegio%TYPE,
  p_localitat IN Localitat.codiLocalitat%TYPE,
  p_edat IN Ciutada.edat%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := 'p_codiAleatori: ' ||  p_codiAleatori || ', p_pais: ' || p_pais ||
    ', p_regio: ' || p_regio || ', p_localitat: ' || p_localitat || ', p_edat: '|| p_edat;

  -- Inserim el nou registre
  INSERT INTO Ciutada (codiAleatori, pais, regio, localitat, edat)
    VALUES (p_codiAleatori, p_pais, p_regio, p_localitat, p_edat);

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta ciutad', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1 THEN
      RSP := 'ERROR: El ciutad ' || p_codiAleatori || ' ja existeix a la base de dades';
    ELSIF SQLCODE = -1400 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta ciutad', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Ciutada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION modificarCiutada (
  p_codiAleatori IN Ciutada.codiAleatori%TYPE,
  p_pais IN Pais.codiPais%TYPE,
  p_regio IN Regio.codiRegio%TYPE,
  p_localitat IN Localitat.codiLocalitat%TYPE,
  p_edat IN Ciutada.edat%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiAleatori: ' ||  p_codiAleatori || ', p_pais: ' || p_pais ||
    ', p_regio: ' || p_regio || ', p_localitat: ' || p_localitat || ', p_edat: '|| p_edat;

  -- Actualitzem les dades del registre
  UPDATE Ciutada SET pais=p_pais, regio=p_regio, localitat=p_localitat, edat=p_edat
    WHERE codiAleatori = p_codiAleatori;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Modificaci ciutad', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el ciutad amb codi:' || p_codiAleatori;
    gravarLog ('Modificaci ciutad', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Modificaci ciutad', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Ciutada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaCiutada (
  p_codiAleatori IN Ciutada.codiAleatori%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;
  HaVotat number;
  JaHaVotat EXCEPTION;
  
BEGIN

  parametresEntrada := 'p_codiAleatori: ' || p_codiAleatori;

  -- Comprovem si ja ha votat doncs no es podr esborrar
  SELECT COUNT(*) INTO HaVotat
    FROM Vot
    WHERE Ciutada = p_codialeatori;
  
  IF HaVotat>0 THEN
    RAISE JaHaVotat;
  END IF;
  
  -- Esborrem el registre
  DELETE Ciutada
    WHERE codiAleatori = p_codiAleatori;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa ciutad', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN JaHaVotat THEN
    RSP := 'ERROR: El ciutad amb codi:' || p_codiAleatori || ' ja ha votat i no es pot esborrar';
    gravarLog ('Baixa ciutad', parametresEntrada, RSP);
    RETURN RSP;

  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el ciutad amb codi:' || p_codiAleatori;
    gravarLog ('Baixa ciutad', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa ciutad', parametresEntrada, RSP);
    RETURN RSP;

END;