ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula Votacio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaVotacio (
  p_titol IN Votacio.titol%TYPE,
  p_descripcioVotacio IN Votacio.descripcioVotacio%TYPE,
  p_dataIniciVot IN Votacio.dataIniciVot%TYPE,
  p_dataFiVot IN Votacio.dataFiVot%TYPE,
  p_dataPublicacioVot IN Votacio.dataPublicacioVot%TYPE,
  p_president IN Votacio.president%TYPE,
  p_adreaWeb IN Votacio.adreaWeb%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := 'p_titol: ' || p_titol || ', p_descripcioVotacio: ' || p_descripcioVotacio ||
    'p_dataIniciVot: ' || p_dataIniciVot || ', p_dataFiVot: ' || p_dataFiVot ||
    'p_dataPublicacioVot: ' || p_dataPublicacioVot || ', p_president: ' || p_president ||
    'p_adreaWeb: ' || p_adreaWeb;

  -- Inserim el nou registre
  INSERT INTO Votacio (titol, descripcioVotacio, dataIniciVot, dataFiVot,
      dataPublicacioVot, president, adreaWeb, nombreVotsTotal, nombrePersonesPodenVotar,
      percentatgeParticipacio, estatVotacio)
    VALUES (p_titol, p_descripcioVotacio, p_dataIniciVot, p_dataFiVot,
      p_dataPublicacioVot, p_president, p_adreaWeb, 0, 0 , 0, 'E');
 
  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta votaci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1400 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquest el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || ' ' || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta votaci', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Votacio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION modificarVotacio (
  p_codiVotacio IN Votacio.codiVotacio%TYPE,
  p_titol IN Votacio.titol%TYPE,
  p_descripcioVotacio IN Votacio.descripcioVotacio%TYPE,
  p_dataIniciVot IN Votacio.dataIniciVot%TYPE,
  p_dataFiVot IN Votacio.dataFiVot%TYPE,
  p_dataPublicacioVot IN Votacio.dataPublicacioVot%TYPE,
  p_president IN Votacio.president%TYPE,
  p_adreaWeb IN Votacio.adreaWeb%TYPE,
  p_estatVotacio IN Votacio.estatVotacio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiVotacio: ' || p_codiVotacio || ', p_titol: ' || p_titol || 
    ', p_descripcioVotacio: ' || p_descripcioVotacio ||
    ', p_dataIniciVot: ' || p_dataIniciVot || ', p_dataFiVot: ' || p_dataFiVot ||
    ', p_dataPublicacioVot: ' || p_dataPublicacioVot || ', p_president: ' || p_president ||
    ', p_adreaWeb: ' || p_adreaWeb || ', p_estatVotacio: ' || p_estatVotacio;

  -- Actualitzem el registre
  UPDATE Votacio SET titol=p_titol, descripcioVotacio=p_descripcioVotacio, dataIniciVot=p_dataIniciVot,
    dataFiVot=p_dataFiVot, dataPublicacioVot=p_dataPublicacioVot, president=p_president,
    adreaWeb=p_adreaWeb, estatVotacio=p_estatVotacio
    WHERE codiVotacio = p_codiVotacio;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b o s l'error controlat
  RSP := 'OK';
  gravarLog ('Modificaci votaci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix la votaci amb codi:' || p_codiVotacio;
    gravarLog ('Modificaci votaci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || ' ' || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Modificaci votaci', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Votacio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaVotacio (
  p_codiVotacio IN Votacio.codiVotacio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiVotacio: ' || p_codiVotacio;

  -- Esborrem el registre
  DELETE Votacio
   WHERE codiVotacio = p_codiVotacio;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa votaci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix la votaci amb codi:' || p_codiVotacio;
    gravarLog ('Baixa votaci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || ' ' || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa votaci', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici del canvi destat de la taula Votacio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION canviEstatVotacio (
  p_codiVotacio IN Votacio.codiVotacio%TYPE,
  p_estatVotacio IN Votacio.estatVotacio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;
  final number:=0;
  JaFinalitzada EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiVotacio: ' || p_codiVotacio || ', p_estatVotacio: ' || p_estatVotacio;

  -- si ja est finalitzada no es pot canviar
  SELECT COUNT(*) INTO final
    FROM Votacio
    WHERE codiVotacio = p_codiVotacio AND estatVotacio = 'F';

  IF final>0 THEN
    RAISE JaFinalitzada;
  END IF;

  -- Actualitzem la votaci amb el nou estat
  UPDATE Votacio SET estatVotacio=p_estatVotacio
    WHERE codiVotacio = p_codiVotacio;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Canvi estat votaci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN JaFinalitzada THEN
    RSP := 'ERROR: La votaci amb codi:' || p_codiVotacio || ' ja est finalitzada i no es pot canviar lestat';
    gravarLog ('Canvi estat votaci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix la votaci amb codi:' || p_codiVotacio;
    gravarLog ('Canvi estat votaci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || ' ' || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Canvi estat votaci', parametresEntrada, RSP);
    RETURN RSP;

END;



-------------------------------------------------------------------------------
-- Creaci trigger quan s'esborra una Votaci 
-- Les votacions es poden esborrar sempre que no hi hagi vots emesos
-- Ho decidim aix per si s'ha creat una per error que es pugui esborrar.
-------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER delete_Votacio
  BEFORE DELETE ON Votacio
  FOR EACH ROW
  WHEN ( old.estatVotacio = 'F')
DECLARE
  anyVotacio number;
  paisVotacio number;
  numVotacionsFinalitzades number;
BEGIN
  anyVotacio := to_char (:old.dataFiVot, 'yyyy');

  -- Busquem el pais on es fa la votaci
  SELECT Cens.pais INTO paisVotacio
    FROM Cens, CensVotacio
    WHERE Cens.codiCens = CensVotacio.codiCens AND CensVotacio.votacio = :old.codiVotacio;

  -- Busquem quantes votacions finalitzades t el pais en l'any
  SELECT nombreVotacFinalitz INTO numVotacionsFinalitzades
    FROM Est1
    WHERE codiPais = paisVotacio AND anyo = anyVotacio;

  -- Actualitzem la taula d'estadstiques
  UPDATE Est1
    SET nombreVotacFinalitz = (numVotacionsFinalitzades-1)
    WHERE codiPais = paisVotacio AND anyo = anyVotacio;
END delete_Votacio;



-------------------------------------------------------------------------------
-- Ens retorna el % de l'opci ms votada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION percOpcioMesVotada (   
  p_votacio IN Votacio.codiVotacio%TYPE
)
RETURN number
IS
 percGuanya number := 0;
BEGIN

  SELECT percentatge INTO percGuanya
  FROM (
  SELECT percentatge 
    FROM Opcio
    WHERE votacio = p_votacio
    ORDER BY nombreVots DESC)
  WHERE rownum<2;

  RETURN percGuanya;
END;


-------------------------------------------------------------------------------
-- Ens retorna el % de l'opci menys votada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION percOpcioMenysVotada (   
  p_votacio IN Votacio.codiVotacio%TYPE
)
RETURN number
IS
 percPerd number := 0;
BEGIN

  SELECT percentatge INTO percPerd
  FROM (
  SELECT percentatge
    FROM Opcio
    WHERE votacio = p_votacio
    ORDER BY nombreVots)
  WHERE rownum<2;
  RETURN percPerd;
END;


-------------------------------------------------------------------------------
-- Ens retorna la diferncia
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION opcioDiferencia (   
  p_votacio IN Votacio.codiVotacio%TYPE
)
RETURN number
IS

BEGIN

  RETURN (percOpcioMesVotada(p_votacio) - percOpcioMenysVotada(p_votacio));
END;


-------------------------------------------------------------------------------
-- Creaci trigger quan s'actualitza o insereix una Votaci 
-------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER update_Votacio
  BEFORE UPDATE ON Votacio
  FOR EACH ROW
  WHEN ( new.estatVotacio = 'F' AND old.estatVotacio <> 'F')

DECLARE
  anyVotacio number;
  paisVotacio VARCHAR2(2);
  codiCensVotacio number;
  tipusCensVotacio VARCHAR2(1);
  votsPossibles number;
  votsFets number;
  perParticipacio FLOAT;
  numVotacions number;
  numVotacionsFinalitzades number;
  percentatgeVot FLOAT;
  localitatMesVots VARCHAR2(50);
  votacioMP VARCHAR2(30);
  possiblesVotants number;
  maiHanVotat number;
  perMHV FLOAT;
  perMaiHaVot FLOAT;
  qVots number;
  CURSOR cursorEst1 (anVotacio number) 
  IS
    SELECT IDEst1, nombreVotacFinalitz
    FROM Est1
    WHERE anyo = anVotacio;
  vID number;
  vVots number;
  existeix number;
  siVoten number;
  eVotMaxPar Votacio.Titol%TYPE;
  eMaxPar number;
  eVotMesDif Votacio.Titol%TYPE;
  eMesDif FLOAT;
  eVotMenysDif Votacio.Titol%TYPE;
  eMenysDif FLOAT;
  eMaxUnCiu number;
  ePaisMaxUnCiu Pais.codiPais%TYPE;
  pDif FLOAT;
  pV number;
  
BEGIN

  -----------------------------------------------------------------------------
  --  Actualitzem Est1
  -----------------------------------------------------------------------------

  -- busquem l'any, el pais i el codi de cens de la votaci
  -- sn els diferents index de les taules d'estadstiques
  anyVotacio := to_char (:old.dataFiVot, 'yyyy');
  SELECT Cens.pais, Cens.CodiCens, Cens.TipusCens INTO paisVotacio, codiCensVotacio, tipusCensVotacio
    FROM Cens, CensVotacio
    WHERE Cens.codiCens = CensVotacio.codiCens AND CensVotacio.votacio = :old.codiVotacio;
  

  -- si el tipus de cens s global cal actualitzar tots els paisos
  IF tipusCensVotacio = 'G' THEN
    OPEN cursorEst1(anyVotacio);
    LOOP FETCH cursorEst1 INTO vID,vVots; 
      EXIT WHEN cursorEst1%NOTFOUND;
        UPDATE Est1
          SET nombreVotacFinalitz = vVots + 1
          WHERE IDEst1 = vID;
    END LOOP; 
    CLOSE cursorEst1;

  ELSE
    -- busquem el nmero actual de votacions finalitzades del pais
    SELECT COUNT(*) INTO existeix
      FROM Est1
      WHERE codiPais = paisVotacio AND anyo = anyVotacio;

    IF existeix>0 THEN
      SELECT nombreVotacFinalitz INTO numVotacionsFinalitzades
        FROM Est1
        WHERE codiPais = paisVotacio AND anyo = anyVotacio;
    ELSE
      numVotacionsFinalitzades := 0;
    END IF;

    -- actualitzem el nmero de votacions finalitzades  
    UPDATE Est1
      SET nombreVotacFinalitz = numVotacionsFinalitzades + 1
      WHERE codiPais = paisVotacio AND anyo = anyVotacio;
  END IF;
  
 
  -----------------------------------------------------------------------------
  --  Actualitzem Est2_3
  -----------------------------------------------------------------------------
 
  -- busquem el numero de votacions de l'any i pel cens de la votaci
  SELECT COUNT(*) INTO existeix
    FROM Est2_3
    WHERE codiCens = codiCensVotacio AND anyo = anyVotacio;

  IF existeix>0 THEN
    SELECT nombreVotacions INTO numVotacions
      FROM Est2_3
      WHERE codiCens = codiCensVotacio AND anyo = anyVotacio;
  ELSE
    numVotacions := 0;
  END IF;
 
  -- busquem el nmero de vots possibles i el nmero de vots fets per trobar el % de participaci
  votsPossibles := 0;
  SELECT COUNT(CensCiutada.ciutada) INTO votsPossibles
    FROM CensCiutada, Cens
    WHERE codiCens = codiCensVotacio AND CensCiutada.cens = Cens.codiCens;
  
  votsFets := 0;
  SELECT COUNT(Vot.ciutada) INTO votsFets
    FROM CensCiutada, Cens, Vot
    WHERE Cens.codiCens = codiCensVotacio AND CensCiutada.cens = Cens.codiCens AND 
          Vot.ciutada = CensCiutada.ciutada AND Vot.haVotat = '1';

  -- busquem el % de participaci
  IF votsPossibles > 0 THEN
    perParticipacio := (votsFets * 100) / votsPossibles;
  ELSE
    perParticipacio := 0;
  END IF;
  
  -- comprovem si ja hi ha registre i sino el creem
  IF existeix>0 THEN
    -- actualitzem el nombre de votacions i el % de participaci
    UPDATE Est2_3
      SET nombreVotacions = (numVotacions + 1), valorMigPercenPartic = perParticipacio
      WHERE codiCens = codiCensVotacio AND anyo = anyVotacio;
  ELSE
    INSERT INTO Est2_3 (codiCens, anyo, nombreVotacions, valorMigPercenPartic)
      VALUES (codiCensVotacio, anyVotacio, (numVotacions + 1),  perParticipacio);
  END IF;



  -----------------------------------------------------------------------------
  --  Actualitzem Est4_5_6
  -----------------------------------------------------------------------------

  -- Mirem el mxim percentatge de votaci que hi ha a l'estadstica
  -- i si la votaci que tanquem en t ms fem el canvi.
  -- guardem tamb el % mxim de votaci per a poder comparar fcilment
  SELECT COUNT(*) INTO existeix
    FROM Est4_5_6;

  IF existeix>0 THEN
    SELECT percVotMesParticip, localitMesVotsEmes,votacioMesParticip, percenCiutMaiHaVot 
      INTO percentatgeVot, localitatMesVots, votacioMP, perMHV
      FROM Est4_5_6
      WHERE anyo = anyVotacio;
  ELSE
    percentatgeVot := 0;
    localitatMesVots := '';
    votacioMP := '';
    perMHV := 0;
  END IF;
  
  IF perParticipacio > percentatgeVot THEN
    percentatgeVot := perParticipacio;
    votacioMP := :new.titol;
  END IF;

  -- Fem servir la taula auxiliar TrEst on es guarda els vots efectuats a cada localitat
  SELECT loc INTO localitatMesVots
  FROM
  (SELECT Localitat.nomLocalitat AS loc
    FROM Localitat,TrEst
    WHERE Localitat.codiLocalitat = TrEst.codiLocalitat
    ORDER BY TrEst.NombreVots DESC)
  WHERE rownum<2;
  
  -- busquem els possibles votants
  possiblesVotants := :old.nombrePersonesPodenVotar;

  -- busquem els que si han votat
  SELECT COUNT(*) INTO siVoten
    FROM trEstVotCiu
    WHERE anyo = anyVotacio;
  maiHanVotat := possiblesVotants - siVoten;
  
  -- busquem el % dels que mai han votat
  IF possiblesVotants > 0 THEN
    perParticipacio := (maiHanVotat * 100) / possiblesVotants;
  ELSE
    perParticipacio := 0;
  END IF;

  -- Comparem amb el valor que hi ha a la taula est
  IF perParticipacio>perMHV THEN
    perMHV := perParticipacio;
  END IF;

  -- actualitzem l'estadstica amb els valors trobats
  IF existeix>0 THEN
    UPDATE Est4_5_6
      SET percenCiutMaiHaVot = perMHV,
          percVotMesParticip = percentatgeVot, 
          votacioMesParticip = votacioMP,
          localitMesVotsEmes = localitatMesVots
      WHERE anyo = anyVotacio;
  ELSE
    INSERT INTO Est4_5_6 (percenCiutMaiHaVot, percVotMesParticip, 
         votacioMesParticip, localitMesVotsEmes, anyo)
      VALUES (perMHV, percentatgeVot, votacioMP, localitatMesVots, anyVotacio);
  END IF;

 
  -----------------------------------------------------------------------------
  --  Actualitzem Est7_8_9_10
  -----------------------------------------------------------------------------
 
  -- Agafem les dades que hi ha ara a la taula i sino les posem a valors max o minims
  SELECT COUNT(*) INTO existeix
    FROM Est7_8_9_10;
  
  IF existeix=0 THEN
    eVotMaxPar := '';
    eMaxPar := 0;
    eVotMesDif := '';
    eMesDif := 0;
    eVotMenysDif := '';
    eMenysDif := 999;
    eMaxUnCiu := '';
    ePaisMaxUnCiu := '';
  ELSE
    SELECT votHistoricaMesParticipacio, maxParticipacio,
           votMesDifOpcioMesMenysVotada, maxDifVots,
           votMenysDifOpcioMesMenysVotada, minDifVots,
           nombreMaximVotsUnCiutada, codiPaisCiutadaMaximVots
      INTO eVotMaxPar, eMaxPar,
           eVotMesDif, eMesDif,
           eVotMenysDif, eMenysDif,
           eMaxUnCiu, ePaisMaxUnCiu
      FROM Est7_8_9_10;
  END IF;
  
  
  -- Comparem la participacio amb la que tenim a la votaci actual
  IF :old.percentatgeParticipacio > eMaxPar THEN
    eMaxPar := :old.percentatgeParticipacio;
    eVotMaxPar := :old.Titol;
  END IF;
  
   
  -- busquem la diferncia entre la opci ms votada i la que menys de la votaci actual
  --  i ho comparem amb la que hi ha a la taula d'estadstica
  pDif:= opcioDiferencia(:old.codiVotacio);
  IF pDif>eMesDif THEN
    eMesDif := pDif;
    eVotMesDif := :old.Titol;
  END IF;
  
  IF pDif<eMenysDif THEN
    eMenysDif := pDif;
    eVotMenysDif := :old.Titol;
  END IF;
  
  -- comprovem que almenys hagi votat una persona
  SELECT COUNT(*) INTO pV
    FROM trEstNumVotacions;
  
  IF pV>0 THEN
    SELECT votacions, pais INTO eMaxUnCiu, ePaisMaxUnCiu
      FROM
        (SELECT votacions,Ciutada.pais AS pais
          FROM trEstNumVotacions,Ciutada
          WHERE trEstNumVotacions.ciutada=Ciutada.codiAleatori
          ORDER BY votacions DESC)
      WHERE rownum<2;
  END IF;  
 
  -- actualitzem l'estadstica amb els valors trobats
  IF existeix=0 THEN
    INSERT INTO Est7_8_9_10 (votHistoricaMesParticipacio,
        maxParticipacio, votMesDifOpcioMesMenysVotada, maxDifVots,
        votMenysDifOpcioMesMenysVotada, minDifVots,
        nombreMaximVotsUnCiutada, codiPaisCiutadaMaximVots)
      VALUES (eVotMaxPar, eMaxPar, eVotMesDif, eMesDif,
        eVotMenysDif, eMenysDif, eMaxUnCiu, ePaisMaxUnCiu);

  ELSE
    UPDATE Est7_8_9_10
      SET votHistoricaMesParticipacio = eVotMaxPar,
        maxParticipacio = eMaxPar,
        votMesDifOpcioMesMenysVotada = eVotMesDif,
        maxDifVots = eMesDif,
        votMenysDifOpcioMesMenysVotada = eVotMenysDif,
        minDifVots = eMenysDif,
        nombreMaximVotsUnCiutada = eMaxUnCiu,
        codiPaisCiutadaMaximVots = ePaisMaxUnCiu;
  END IF;
 
END update_Votacio;

-------------------------------------------------------------------------------
-- Definici de getCodiVotacio
--  per obtenir el codi d'una votaci amb el ttol de la votaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION getCodiVotacio (   
  p_titol IN Votacio.titol%TYPE
)
RETURN number
IS
 codi number := 0;

BEGIN

  SELECT codiVotacio INTO codi
  FROM Votacio
  WHERE titol = p_titol;

  RETURN codi;
END;