ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula Vot
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaVot (
  p_ciutada IN Vot.ciutada%TYPE,
  p_votacio IN Vot.votacio%TYPE,
  p_opcio IN Vot.opcio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  jaHaVotat number;
  JaExisteix EXCEPTION;
  votsFets number;
  totalVotants number;
  percentatgeVots number;
  estatVotacio VARCHAR2(1 CHAR);
  VotFinal EXCEPTION;
  votsOpcio number;
  percOpcio number;
  codiOp number;
  CURSOR opcions (vVotacio Opcio.votacio%TYPE)
  IS
    SELECT codiOpcio,nombreVots
    FROM Opcio
    WHERE votacio = vVotacio;
  dataVot date;
  nVots number;
  poblacio Localitat.codiLocalitat%TYPE;
  anyVotacio number;
  existeix number;
  pV number;
  
BEGIN

  parametresEntrada := 'p_ciutada: ' || p_ciutada || ', p_votacio: ' || p_votacio || 
    ', p_opcio: ' || p_opcio;

  -- Comprovem que la votaci no est finalitzada
  SELECT NombreVotsTotal, NombrePersonesPodenVotar, EstatVotacio INTO votsFets, totalVotants, estatVotacio
    FROM Votacio
    WHERE codiVotacio = p_votacio;

  IF estatVotacio='F' THEN
    RAISE VotFinal;
  END IF;

  -- Comprovem que no hagi votat el ciutada en aquesta votaci
  SELECT COUNT(*) INTO jaHaVotat
    FROM Vot
    WHERE ciutada = p_ciutada AND votacio = p_votacio AND haVotat > 0;

  IF jaHaVotat>0 THEN
    RAISE JaExisteix;
  END IF;

  IF p_opcio IS NULL THEN
  
    -- Inserim el nou registre indicant que no ha votat
    INSERT INTO Vot (ciutada, votacio, opcio, haVotat)
      VALUES (p_ciutada, p_votacio, NULL, 0);
  
  ELSE
    -- Actualitzem el vot
    UPDATE Vot 
      SET opcio=p_opcio,haVotat=1
      WHERE ciutada=p_ciutada AND votacio=p_votacio;

    -- Incrementem el nombre de vots efectuats a la votacio
    votsFets := votsFets + 1;  

    -- I actualitzem la taula Opcio
    SELECT NombreVots INTO votsOpcio
      FROM Opcio
      WHERE codiOpcio = p_opcio;
    votsOpcio := votsOpcio + 1;
    percOpcio := (votsOpcio * 100) / votsFets;
    UPDATE Opcio
      SET nombrevots=votsOpcio
      WHERE codiOpcio = p_opcio;
       
    -- Actualitzem els percentatges de totes les opcions
    OPEN opcions(p_votacio);
    LOOP FETCH opcions INTO codiOp, votsOpcio; 
      EXIT WHEN opcions%NOTFOUND;
        percOpcio := (votsOpcio * 100) / votsFets;
        UPDATE Opcio
          SET percentatge=percOpcio
          WHERE codiOpcio = codiOp;
      END LOOP; 
    CLOSE opcions;

    -- Actualitzem els vots per any i localitats
    SELECT dataFiVot INTO dataVot
      FROM Votacio
      WHERE codiVotacio = p_votacio;
    anyVotacio := to_char (dataVot, 'yyyy');
    SELECT localitat INTO poblacio
      FROM Ciutada
      WHERE codiAleatori = p_ciutada;
    SELECT COUNT(*) INTO existeix
      FROM TrEst
      WHERE anyo = anyVotacio AND codiLocalitat = poblacio;
    IF existeix>0 THEN
      SELECT NombreVots INTO nVots
        FROM TrEst
        WHERE anyo = anyVotacio AND codiLocalitat = poblacio;
      nVots := nVots + 1;
      UPDATE TrEst
        SET NombreVots=nVots
        WHERE anyo = anyVotacio AND codiLocalitat = poblacio;
    ELSE
      INSERT INTO TrEst (anyo, codiLocalitat, NombreVots)
        VALUES (anyvotacio,poblacio,1);
      
    END IF;

    -- Marquem que aquest ciutad ja ha votat aquest any per l'estadstica 6
    SELECT COUNT(*) INTO existeix
      FROM trEstVotCiu
      WHERE ciutada = p_ciutada AND anyVotacio = anyo;

    IF existeix=0 THEN  
      INSERT INTO trEstVotCiu (ciutada, anyo)
        VALUES (p_ciutada, anyVotacio);
    END IF;

    -- Incrementem el nombre de votacions del ciutada per l'estadstica
    SELECT COUNT(*) INTO existeix
      FROM trEstNumVotacions
      WHERE ciutada = p_ciutada;
    
    IF existeix>0 THEN  
      SELECT votacions INTO pV
        FROM trEstNumVotacions
        WHERE ciutada = p_ciutada;
    ELSE
      pV := 0;
    END IF;
    
    IF pV=0 THEN
      pV := 1;
      INSERT INTO trEstNumVotacions (ciutada, votacions)
        VALUES (p_ciutada, pV);
    ELSE
      pV := pV + 1;
      UPDATE trEstNumVotacions
        SET votacions=pV
        WHERE ciutada=p_ciutada;
    END IF;
  END IF;

  -- Actualitzem els vots efectuats a la votaci
  IF totalVotants=0 THEN
    percentatgeVots := 0;
  ELSE
    percentatgeVots := (votsFets * 100) / totalVotants;
  END IF;
  
  UPDATE Votacio
    SET NombreVotsTotal=votsFets, PercentatgeParticipacio=percentatgeVots
    WHERE codiVotacio = p_votacio;
    
  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta vot', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN VotFinal THEN
    RSP := 'ERROR: La votaci amb codi:' || p_votacio || ' ja est finalitzada i no es pot votar.';
    gravarLog ('Alta vot', parametresEntrada, RSP);
    RETURN RSP;

  WHEN JaExisteix THEN
    RSP := 'ERROR: El ciutada amb codi:' || p_ciutada || ' ja ha votat en la votaci: ' || p_votacio;
    gravarLog ('Alta vot', parametresEntrada, RSP);
    RETURN RSP;

  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1400 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta vot', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Vot
-- No es permet modificar un vot una vegada s'ha ems
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Vot
-- Noms es permet esborrar un vot si no s'ha ems, 
--  o sigui quan s'ha creat al donar d'alta el cens a la votaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaVot (
  p_votacio IN Vot.votacio%TYPE,
  p_ciutada IN Vot.ciutada%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  jaHaVotat number;
  JaExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_votacio: ' || p_votacio || ', p_ciutada: ' || p_ciutada;

  -- Comprovem que no hagi votat el ciutada en aquesta votaci
  SELECT COUNT(*) INTO jaHaVotat
    FROM Vot
    WHERE ciutada = p_ciutada AND votacio = p_votacio AND haVotat > 0;

  IF jaHaVotat>0 THEN
    RAISE JaExisteix;
  END IF;

  -- Esborrem el registre
  DELETE FROM Vot
    WHERE Ciutada = p_ciutada AND Votacio = p_votacio;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa vot', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN JaExisteix THEN
    RSP := 'ERROR: El ciutada amb codi:' || p_ciutada || ' ja ha votat en aquesta votaci i no es pot esborrar.';
    gravarLog ('Baixa vot', parametresEntrada, RSP);

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa vot', parametresEntrada, RSP);
    RETURN RSP;

END;