ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula CensVotacio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaCensVotacio (
  p_cens IN Cens.codiCens%TYPE,
  p_Votacio IN Votacio.codiVotacio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  CURSOR vots (vCens Cens.codiCens%TYPE)
  IS
    SELECT ciutada
    FROM CensCiutada
    WHERE cens = vCens;
  vCiutada Ciutada.codiAleatori%TYPE;
  votsPossibles number;
  
BEGIN

  parametresEntrada := 'p_cens: ' || p_cens || ', p_votacio: ' || p_votacio;

  -- Inserim el nou registre
  INSERT INTO CensVotacio (codiCens, Votacio)
    VALUES (p_cens, p_votacio);

  -- Creem els registre de la taula Vot per a tots els ciutadans del cens
  --    amb els valors haVotat = 0
  votsPossibles := 0;
  OPEN vots(p_cens);
  LOOP FETCH vots INTO vCiutada; 
     EXIT WHEN vots%NOTFOUND;
       RSP := altaVot( vCiutada, p_votacio, NULL);
       votsPossibles := votsPossibles + 1;
     END LOOP; 
  CLOSE vots;

  -- Actualitzem els vots possibles de la votaci
  UPDATE Votacio
    SET NombrePersonesPodenVotar=votsPossibles
    WHERE codiVotacio = p_votacio;
    
  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta cens votaci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1400 THEN
      RSP := 'ERROR: El cens i la votaci sn camps obligatoris';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta cens votaci', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula CensVotacio
-- en aquest cas no hi ha modificaci doncs els dos camps formen l'index
-- Per a fer canvis cal esborrar el registre i fer-lo de nou.
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula CensVotacio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaCensVotacio (
  p_cens IN Cens.codiCens%TYPE,
  p_Votacio IN Votacio.codiVotacio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;
  HiHaVots EXCEPTION;
  CURSOR vots (vVotacio Votacio.codiVotacio%TYPE)
  IS
    SELECT ciutada
    FROM Vot
    WHERE Votacio = vVotacio;
  vCiutada Ciutada.codiAleatori%TYPE;
  votsFets number;

BEGIN

  parametresEntrada := 'p_cens: ' || p_cens || ', p_Votacio: ' || p_Votacio;

  -- Mirem si hi ha vots fets doncs no es podr esborrar
  SELECT COUNT(Ciutada) INTO votsFets
    FROM Vot
    WHERE votacio = p_Votacio;

  -- Si n'hi ha llancem una excepci
  IF votsFets>0 THEN
    RAISE HiHaVots;
  END IF;
 
  -- Esborrem el registre
  DELETE CensVotacio
   WHERE codiCens = p_cens AND Votacio = p_Votacio;

  -- Si no troba el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Esborrem els vots per fer
  OPEN vots(p_Votacio);
  LOOP FETCH vots INTO vCiutada; 
     EXIT WHEN vots%NOTFOUND;
       RSP := baixaVot( p_Votacio, vCiutada);
     END LOOP; 
  CLOSE vots;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa cens votaci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN HiHaVots THEN
    RSP := 'ERROR: Hi ha vots emesos a la Votacio amb codi: ' || p_Votacio;
    gravarLog ('Baixa cens votaci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el cens amb codi: ' || p_cens || ' o la Votacio amb codi: ' || p_Votacio;
    gravarLog ('Baixa cens votaci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa cens votaci', parametresEntrada, RSP);
    RETURN RSP;

END;