ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;


-------------------------------------------------------------------------------
-- Ens retorna el percentatge de participaci d'una votaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION perParticipacioVotacio (   
  p_votacio IN Votacio.codiVotacio%TYPE
)
RETURN VARCHAR2
IS
 perc number := -1;
 percChar VARCHAR2(10) := '-';

BEGIN

  SELECT percentatgeParticipacio INTO perc
  FROM Votacio
  WHERE codiVotacio = p_votacio AND estatVotacio = 'F';

  IF perc<>-1 THEN
    percChar := to_char(perc, '999.99') || '%';
  END IF;

  RETURN percChar;
END;


-------------------------------------------------------------------------------
-- Ens retorna la opci que ha guanyat a una votaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION opcioMesVotadaVotacio (   
  p_votacio IN Votacio.codiVotacio%TYPE
)
RETURN VARCHAR2
IS
 opcioGuanya VARCHAR2(100) := '-';

BEGIN

  SELECT opcio INTO opcioGuanya
  FROM (
  SELECT Opcio.descripcioOpcio AS opcio
    FROM Votacio, Opcio
    WHERE Votacio.codiVotacio = p_votacio AND Votacio.codiVotacio = Opcio.votacio AND 
          Votacio.estatVotacio = 'F'
    ORDER BY Opcio.nombreVots DESC)
  WHERE rownum<2;

  RETURN opcioGuanya;
END;


-------------------------------------------------------------------------------
-- Ens retorna la opci que ha perdut a una votaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION opcioMenysVotadaVotacio (   
  p_votacio IN Votacio.codiVotacio%TYPE
)
RETURN VARCHAR2
IS
 opcioPerd VARCHAR2(100) := '-';

BEGIN

  SELECT opcio INTO opcioPerd
  FROM (
  SELECT Opcio.descripcioOpcio AS opcio
    FROM Votacio, Opcio
    WHERE Votacio.codiVotacio = p_votacio AND Votacio.codiVotacio = Opcio.votacio AND 
          Votacio.estatVotacio = 'F'
    ORDER BY Opcio.nombreVots)
  WHERE rownum<2;

  RETURN opcioPerd;
END;


-------------------------------------------------------------------------------
-- Ens retorna els % de vots de la opci que ha guanyat a una votaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION perVotsOpGuanyaVotacio (   
  p_votacio IN Votacio.codiVotacio%TYPE
)
RETURN VARCHAR2
IS
 perc number := -1;
 percChar VARCHAR2(10) := '-';
BEGIN

  SELECT opcio INTO perc
  FROM (
  SELECT Opcio.percentatge AS opcio
    FROM Votacio, Opcio
    WHERE Votacio.codiVotacio = p_votacio AND Votacio.codiVotacio = Opcio.votacio AND 
          Votacio.estatVotacio = 'F'
    ORDER BY Opcio.nombreVots DESC)
  WHERE rownum<2;

  IF perc<>-1 THEN
    percChar := to_char(perc, '999.99') || '%';
  END IF;

  RETURN percChar;
END;


-------------------------------------------------------------------------------
-- Definici de la consulta a. 
-- Donat un pais retorna les votacions que s'hi han produt
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION consVotacionsPais (
  p_resultset IN OUT sys_refcursor,
  p_pais IN Pais.codiPais%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SensePais EXCEPTION;

BEGIN

  parametresEntrada := 'p_pais: ' || p_pais;

  IF p_pais IS NULL THEN
    RAISE SensePais;
  ELSE
    OPEN p_resultset FOR
      SELECT Votacio.titol, Votacio.dataIniciVot, Votacio.dataFiVot, 
             Votacio.nombrePersonesPodenVotar, Votacio.estatVotacio,
             perParticipacioVotacio (Votacio.codiVotacio) AS percentatgeparticipacio,
             opcioMesVotadaVotacio (Votacio.codiVotacio) AS opciomesvotada,
             perVotsOpGuanyaVotacio (Votacio.codiVotacio) AS percentatgevotsguanyador
      FROM Votacio, Cens, CensVotacio 
      WHERE Votacio.codiVotacio = CensVotacio.votacio AND
            CensVotacio.codicens = Cens.codiCens AND
            (Cens.tipusCens = 'G' OR 
            (Cens.tipusCens = 'N' AND Cens.pais = p_pais))
      ORDER BY Votacio.dataIniciVot DESC;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Consulta votacions per pais', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN SensePais THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: Cal informar el pais del que volem les votacions';
    gravarLog ('Consulta votacions per pais', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Consulta votacions per pais', parametresEntrada, RSP);
    RETURN RSP;

END;



-------------------------------------------------------------------------------
-- Definici de la consulta b. 
-- retorna les 10 votacions on hi ha hagut ms diferncia % entre l'opci ms i
-- la menys votada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION cons10VotacionsMesDif (   
  p_resultset IN OUT sys_refcursor
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := '';

  OPEN p_resultset FOR
    SELECT * FROM
      (SELECT Votacio.titol AS descripcioVotacio, 
              Votacio.dataIniciVot AS dataIniciVotacio, 
              Votacio.dataFiVot AS dataFiVotacio, 
              opcioMesVotadaVotacio (Votacio.codiVotacio) AS OpcioMesVotada, 
              percOpcioMesVotada (Votacio.codiVotacio) AS percOpcioMesVotada,
              opcioMenysVotadaVotacio (Votacio.codiVotacio) AS OpcioMenysVotada, 
              percOpcioMenysVotada (Votacio.codiVotacio) AS percOpcioMenysVotada,
              opcioDiferencia (Votacio.codiVotacio) AS diferencia
       FROM Votacio
       WHERE Votacio.estatVotacio = 'F'
       ORDER BY diferencia DESC)
    WHERE rownum < 11;
 
  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Consulta 10 votacions amb ms diferncia', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN OTHERS THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Consulta 10 votacions amb ms diferncia', parametresEntrada, RSP);
    RETURN RSP;

END;



-------------------------------------------------------------------------------
-- Definici de la consulta c. 
-- retorna les 10 votacions on hi ha hagut menys diferncia % entre l'opci ms i
-- la menys votada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION cons10VotacionsMenysDif (   
  p_resultset IN OUT sys_refcursor
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := '';

  OPEN p_resultset FOR
    SELECT * FROM
      (SELECT Votacio.titol AS descripcioVotacio, 
              Votacio.dataIniciVot AS dataIniciVotacio, 
              Votacio.dataFiVot AS dataFiVotacio, 
              opcioMesVotadaVotacio (Votacio.codiVotacio) AS OpcioMesVotada, 
              percOpcioMesVotada (Votacio.codiVotacio) AS percOpcioMesVotada,
              opcioMenysVotadaVotacio (Votacio.codiVotacio) AS OpcioMenysVotada, 
              percOpcioMenysVotada (Votacio.codiVotacio) AS percOpcioMenysVotada,
              opcioDiferencia (Votacio.codiVotacio) AS diferencia
       FROM Votacio
       WHERE Votacio.estatVotacio = 'F'
       ORDER BY diferencia)
    WHERE rownum < 11;
 
  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Consulta 10 votacions amb menys diferncia', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN OTHERS THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Consulta 10 votacions amb menys diferncia', parametresEntrada, RSP);
    RETURN RSP;

END;



-------------------------------------------------------------------------------
-- Definici de la consulta d. 
-- Donat un ciutad retorna els censos als que pertany
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION consCensosCiutada (   
  p_resultset IN OUT sys_refcursor,
  p_ciutada IN Ciutada.codiAleatori%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SenseCiutada EXCEPTION;

BEGIN

  parametresEntrada := 'p_ciutada: ' || p_ciutada;

  IF p_ciutada IS NULL THEN
    RAISE SenseCiutada;
  ELSE
    OPEN p_resultset FOR
      SELECT Cens.codicens AS codicens,
             Cens.descripcio AS descripcio,
             Cens.pais AS pais,
             Cens.regio AS regio,
             Cens.localitat AS localitat,
             Cens.tipusCens AS tipusCens
      FROM Cens, CensCiutada
      WHERE Cens.codiCens = CensCiutada.cens AND CensCiutada.ciutada = p_ciutada;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Consulta censos per ciutad', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN SenseCiutada THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: Cal informar el ciutad del que volem els censos';
    gravarLog ('Consulta censos per ciutad', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Consulta censos per ciutad', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la consulta e. 
-- Donat un ciutad retorna les votacions en que ha participat i les que no
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION consVotacionsCiutada (   
  p_resultset IN OUT sys_refcursor,
  p_ciutada IN Ciutada.codiAleatori%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SenseCiutada EXCEPTION;

BEGIN

  parametresEntrada := 'p_ciutada: ' || p_ciutada;

  IF p_ciutada IS NULL THEN
    RAISE SenseCiutada;
  ELSE
    OPEN p_resultset FOR
      SELECT Votacio.*, Vot.haVotat
      FROM Votacio, Vot
      WHERE Votacio.codiVotacio = Vot.votacio AND Vot.ciutada = p_ciutada;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Consulta votacions per ciutad', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN SenseCiutada THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: Cal informar el ciutad del que volem les votacions';
    gravarLog ('Consulta votacions per ciutad', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Retorna un resulset sense registres
    OPEN p_resultset FOR SELECT * FROM dual WHERE dummy IS NULL;

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Consulta votacions per ciutad', parametresEntrada, RSP);
    RETURN RSP;

END;
