ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de l'estadstica 1. 
-- Donat un pais i un any retorna el nombre 
-- de votacions finalitzades que s'hi han produt
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estVotacionsPaisAny (
  p_pais IN Pais.codiPais%TYPE,
  p_any IN Number
) RETURN Number
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SensePais EXCEPTION;
  eRes number;

BEGIN

  parametresEntrada := 'p_pais: ' || p_pais || ', p_any: ' || p_any;

  eRes := -1;
  IF p_pais IS NULL OR p_any IS NULL THEN
    RAISE SensePais;
  ELSE
    SELECT nombreVotacFinalitz INTO eRes
    FROM Est1
    WHERE codiPais = p_pais AND anyo = p_any;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.1- Votacions per pas i any', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN SensePais THEN
    -- Gravem l'error
    RSP := 'ERROR: Cal informar el pais i lany dels que volem les votacions';
    gravarLog ('Est.1- Votacions per pas i any', parametresEntrada, RSP);
    RETURN eRes;

  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.1- Votacions per pas i any', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 2. 
-- Donat un cens i un any retorna el valor mitj del
-- percentatge de participaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estParticipacioCensAny (
  p_cens IN Cens.codiCens%TYPE,
  p_any IN Number
) RETURN Float
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SenseCens EXCEPTION;
  eRes float;

BEGIN

  parametresEntrada := 'p_cens: ' || p_cens || ', p_any: ' || p_any;

  eRes := -1;
  IF p_cens IS NULL OR p_any IS NULL THEN
    RAISE SenseCens;
  ELSE
    SELECT valorMigPercenPartic INTO eRes
    FROM Est2_3
    WHERE codiCens = p_cens AND anyo = p_any;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.2- Participaci per cens i any', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN SenseCens THEN
    -- Gravem l'error
    RSP := 'ERROR: Cal informar el cens i lany dels que volem la participaci';
    gravarLog ('Est.2- Participaci per cens i any', parametresEntrada, RSP);
    RETURN eRes;

  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.2- Participaci per cens i any', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 3. 
-- Donat un cens i un any retorna el nombre 
-- de votacions finalitzades que s'hi han produt
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estVotacionsCensAny (
  p_cens IN Cens.codiCens%TYPE,
  p_any IN Number
) RETURN Number
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SenseCens EXCEPTION;
  eRes number;

BEGIN

  parametresEntrada := 'p_cens: ' || p_cens || ', p_any: ' || p_any;

  eRes := -1;
  IF p_cens IS NULL OR p_any IS NULL THEN
    RAISE SenseCens;
  ELSE
    SELECT nombreVotacions INTO eRes
    FROM Est2_3
    WHERE codiCens = p_cens AND anyo = p_any;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.3- Votacions per cens i any', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN SenseCens THEN
    -- Gravem l'error
    RSP := 'ERROR: Cal informar el cens i lany dels que volem les votacions';
    gravarLog ('Est.3- Votacions per cens i any', parametresEntrada, RSP);
    RETURN eRes;

  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.3- Votacions per cens i any', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 4. 
-- Donat un any retorna la votaci que ms participaci ha tingut
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estVotacioMesParticipAny (
  p_any IN Number
) RETURN Est4_5_6.votacioMesParticip%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SenseAny EXCEPTION;
  eRes Est4_5_6.votacioMesParticip%TYPE;

BEGIN

  parametresEntrada := 'p_any: ' || p_any;

  eRes := '';
  IF p_any IS NULL THEN
    RAISE SenseAny;
  ELSE
    SELECT votacioMesParticip INTO eRes
    FROM Est4_5_6
    WHERE anyo = p_any;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.4- Votaci ms participaci per any', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN SenseAny THEN
    -- Gravem l'error
    RSP := 'ERROR: Cal informar lany dels que volem la participaci';
    gravarLog ('Est.4- Votaci ms participaci per any', parametresEntrada, RSP);
    RETURN eRes;

  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.4- Votaci ms participaci per any', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 5. 
-- Donat un any retorna la localitat que ms vots han ems els seus ciutadans
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estLocalitatMesVotsAny (
  p_any IN Number
) RETURN Est4_5_6.localitMesVotsEmes%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SenseAny EXCEPTION;
  eRes Est4_5_6.localitMesVotsEmes%TYPE;

BEGIN

  parametresEntrada := 'p_any: ' || p_any;

  eRes := '';
  IF p_any IS NULL THEN
    RAISE SenseAny;
  ELSE
    SELECT localitMesVotsEmes INTO eRes
    FROM Est4_5_6
    WHERE anyo = p_any;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.5- Localitat ms vots per any', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN SenseAny THEN
    -- Gravem l'error
    RSP := 'ERROR: Cal informar lany dels que volem la localitat amb ms vots';
    gravarLog ('Est.5- Localitat ms vots per any', parametresEntrada, RSP);
    RETURN eRes;

  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.5- Localitat ms vots per any', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 6. 
-- Donat un any retorna el percentatge de ciutadans que mai ha votat
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estPercentMaiVotatAny (
  p_any IN Number
) RETURN Float
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  SenseAny EXCEPTION;
  eRes float;

BEGIN

  parametresEntrada := 'p_any: ' || p_any;

  eRes := -1;
  IF p_any IS NULL THEN
    RAISE SenseAny;
  ELSE
    SELECT percenCiutMaiHaVot INTO eRes
    FROM Est4_5_6
    WHERE anyo = p_any;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.6- % mai ha votat per any', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN SenseAny THEN
    -- Gravem l'error
    RSP := 'ERROR: Cal informar lany dels que volem el % que mai ha votat';
    gravarLog ('Est.6- % mai ha votat per any', parametresEntrada, RSP);
    RETURN eRes;

  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.6- % mai ha votat per any', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 7. 
-- Retorna la votaci que histricament ha tingut ms participaci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estVotacioMesParticip 
RETURN Est7_8_9_10.votHistoricaMesParticipacio%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  eRes Est7_8_9_10.votHistoricaMesParticipacio%TYPE;

BEGIN

  parametresEntrada := '-';

  eRes := '';

  SELECT votHistoricaMesParticipacio INTO eRes
  FROM Est7_8_9_10;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.7- Votaci ms participaci', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.7- Votaci ms participaci', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 8. 
-- Retorna la votaci que histricament ha tingut ms diferncia % 
-- entre l'opci ms votada i la menys
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estVotacioMesDif 
RETURN Est7_8_9_10.votMesDifOpcioMesMenysVotada%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  eRes Est7_8_9_10.votMesDifOpcioMesMenysVotada%TYPE;

BEGIN

  parametresEntrada := '-';

  eRes := '';

  SELECT votMesDifOpcioMesMenysVotada INTO eRes
  FROM Est7_8_9_10;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.8- Votaci ms diferncia vots', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.8- Votaci ms diferncia vots', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 9. 
-- Retorna la votaci que histricament ha tingut menys diferncia % 
-- entre l'opci ms votada i la menys
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estVotacioMenysDif 
RETURN Est7_8_9_10.votMenysDifOpcioMesMenysVotada%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  eRes Est7_8_9_10.votMenysDifOpcioMesMenysVotada%TYPE;

BEGIN

  parametresEntrada := '-';

  eRes := '';

  SELECT votMenysDifOpcioMesMenysVotada INTO eRes
  FROM Est7_8_9_10;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.9- Votaci menys diferncia vots', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.9- Votaci menys diferncia vots', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 10-A. 
-- Retorna el nombre mxim de votacions en que ha participat una mateixa persona
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estMaxVotacionsUnCiutada 
RETURN Number
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  eRes number;

BEGIN

  parametresEntrada := '-';

  eRes := -1;

  SELECT nombreMaximVotsUnCiutada INTO eRes
  FROM Est7_8_9_10;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.10A- Mxim votacions un ciutad', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.10A- Mxim votacions un ciutad', parametresEntrada, RSP);
    RETURN eRes;

END;



-------------------------------------------------------------------------------
-- Definici de l'estadstica 10-B. 
-- Retorna el pas on viu el ciutad amb el nombre mxim de votacions 
-- en que ha participat una mateixa persona
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION estPaisMaxVotacionsUnCiutada 
RETURN Est7_8_9_10.codiPaisCiutadaMaximVots%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  eRes Est7_8_9_10.codiPaisCiutadaMaximVots%TYPE;

BEGIN

  parametresEntrada := '-';

  eRes := '';

  SELECT codiPaisCiutadaMaximVots INTO eRes
  FROM Est7_8_9_10;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Est.10B- Mxim votacions un ciutad', parametresEntrada, RSP);

  RETURN eRes;

EXCEPTION
  WHEN OTHERS THEN

    -- Gravem l'error
    RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    gravarLog ('Est.10B- Mxim votacions un ciutad', parametresEntrada, RSP);
    RETURN eRes;

END;

