

CREATE USER VOTACIONS
IDENTIFIED BY VOTACIONS_ESQ
DEFAULT TABLESPACE users
QUOTA UNLIMITED ON users
TEMPORARY TABLESPACE temp;


GRANT CREATE SESSION, CREATE VIEW,
      CREATE PROCEDURE, EXECUTE ANY PROCEDURE,
      CREATE TRIGGER, CREATE SEQUENCE, 
      CREATE TABLE, UPDATE ANY TABLE, INSERT ANY TABLE, SELECT ANY TABLE
      TO VOTACIONS;
COMMIT;  


-------------------------------------------------------------------------------
-- Primer definim les taules auxiliars que no tenen foreign keys:
--   EstatVotacio, TipusCens, Pais, Regio, Localitat
--
-- A continuaci definim les taules mestres:
--   Cens, Ciutada, Votacio, Opcio, Vot
--
-- Passem a definir les relacions entre les taules mestres
--   CensCiutada, CensVotacio
--
-- Finalment definim les taules de log i estadistiques 
-- i les que fem servir per calcular les estadstiques
--   RegistreLog, Est1, Est2_3, trEst, trEstVotCiu, trEstNumVotacions, Est4_5_6, Est7_8_9_10
--
-------------------------------------------------------------------------------

ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;


-------------------------------------------------------------------------------
-- Creaci taula EstatVotacio
--
-- Donat que noms hi ha 4 valors i sn fixes decidim que
-- no s'ha de fer altes ni baixes ni modificacions a aquesta taula.
-------------------------------------------------------------------------------
CREATE TABLE EstatVotacio(
  codiEstat VARCHAR2(1 CHAR) CONSTRAINT PK_codiEstat PRIMARY KEY,
  nomEstat VARCHAR2(25 CHAR) CONSTRAINT NN_EstatVotacio_nomEstat NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula TipusCens 
--
-- Donat que noms hi ha 4 valors i sn fixes decidim que
-- no s'ha de fer altes ni baixes ni modificacions a aquesta taula.
-------------------------------------------------------------------------------
CREATE TABLE TipusCens(
  codiTipus VARCHAR2(1 CHAR) CONSTRAINT PK_codiTipus PRIMARY KEY,
  nomTipus VARCHAR2(15 CHAR) CONSTRAINT NN_TipusCens_nomTipus NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Pais 
--
-- La taula de paisos l'agafem de la ISO 3166-1 alfa 1
-------------------------------------------------------------------------------
CREATE TABLE Pais(
  codiPais VARCHAR2(2CHAR) CONSTRAINT PK_codiPais PRIMARY KEY,
  nomPais VARCHAR2(50CHAR) CONSTRAINT NN_Pais_nomPais NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Regio 
-------------------------------------------------------------------------------
CREATE TABLE Regio(
  codiRegio NUMBER CONSTRAINT PK_codiRegio PRIMARY KEY,
  nomRegio VARCHAR2(50CHAR) CONSTRAINT NN_Regio_nomRegio NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Localitat 
-------------------------------------------------------------------------------
CREATE TABLE Localitat(
  codiLocalitat NUMBER CONSTRAINT PK_codiLocalitat PRIMARY KEY,
  nomLocalitat VARCHAR2(50CHAR) CONSTRAINT NN_Localitat_nomLocalitat NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Cens
-------------------------------------------------------------------------------
CREATE TABLE Cens(
  codiCens NUMBER CONSTRAINT PK_codiCens PRIMARY KEY,
  descripcio VARCHAR2(50 CHAR) CONSTRAINT NN_Cens_descripcio NOT NULL,
  pais CONSTRAINT FK1_Cens REFERENCES Pais(codiPais),
  regio CONSTRAINT FK2_Cens REFERENCES Regio(codiRegio),
  localitat CONSTRAINT FK3_Cens REFERENCES Localitat(codiLocalitat),
  tipusCens CONSTRAINT FK4_Cens REFERENCES TipusCens(codiTipus)
		CONSTRAINT NN_Cens_tipusCens NOT NULL
);



-------------------------------------------------------------------------------
-- Creaci taula Ciutada
-------------------------------------------------------------------------------
CREATE TABLE Ciutada(
  codiAleatori NUMBER CONSTRAINT PK_codiAleatori PRIMARY KEY,
  pais CONSTRAINT FK1_Ciutada REFERENCES Pais(codiPais)
		CONSTRAINT NN_Ciutada_pais NOT NULL,
  regio CONSTRAINT FK2_Ciutada REFERENCES Regio(codiRegio)
		CONSTRAINT NN_Ciutada_regio NOT NULL,
  localitat CONSTRAINT FK3_Ciutada REFERENCES Localitat(codiLocalitat)
		CONSTRAINT NN_Ciutada_localitat NOT NULL,
  edat NUMBER CONSTRAINT NN_Ciutada_edat NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Votacio 
-------------------------------------------------------------------------------
CREATE TABLE Votacio(
  codiVotacio NUMBER CONSTRAINT PK_codiVotacio PRIMARY KEY, 
  president VARCHAR2(50 CHAR) CONSTRAINT NN_Votacio_president NOT NULL,
  titol VARCHAR2(30 CHAR) CONSTRAINT NN_Votacio_titol NOT NULL,
  descripcioVotacio VARCHAR2(100 CHAR) CONSTRAINT NN_Votacio_descripcioVotacio NOT NULL,
  dataIniciVot DATE CONSTRAINT NN_Votacio_dataIniciVot NOT NULL,
  dataFiVot DATE CONSTRAINT NN_Votacio_dataFiVot NOT NULL,
  dataPublicacioVot DATE CONSTRAINT NN_Votacio_dataPublicacioVot NOT NULL,
  adreaWeb VARCHAR2(100 CHAR),
  nombreVotsTotal NUMBER CONSTRAINT NN_Votacio_nombreVotsTotal NOT NULL,
  nombrePersonesPodenVotar NUMBER CONSTRAINT NN_Votacio_nPersPodenVotar NOT NULL,
  percentatgeParticipacio FLOAT CONSTRAINT NN_Votacio_percPartic NOT NULL,
  estatVotacio CONSTRAINT FK1_estatVotacio REFERENCES EstatVotacio(codiEstat) 
               CONSTRAINT NN_estat_Votacio NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Opcio
-------------------------------------------------------------------------------
CREATE TABLE Opcio(
  codiOpcio NUMBER CONSTRAINT PK_codiOpcio PRIMARY KEY,
  votacio CONSTRAINT FK_Opcio REFERENCES Votacio(codiVotacio)
		CONSTRAINT NN_Opcio_votacio NOT NULL,
  nombreVots NUMBER CONSTRAINT NN_Opcio_nombreVots NOT NULL,
  percentatge FLOAT CONSTRAINT NN_Opcio_percentatge NOT NULL,
  descripcioOpcio VARCHAR(100 CHAR) CONSTRAINT NN_Opcio_descripcioOpcio NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Vot
-------------------------------------------------------------------------------
CREATE TABLE Vot(
  ciutada CONSTRAINT FK1_Vot REFERENCES Ciutada(codiAleatori)
		CONSTRAINT NN_Vot_ciutada NOT NULL,
  votacio CONSTRAINT FK2_Vot REFERENCES Votacio(codiVotacio)
		CONSTRAINT NN_Vot_votacio NOT NULL,
  opcio CONSTRAINT FK3_Vot REFERENCES Opcio(codiOpcio),
  haVotat CHAR CONSTRAINT NN_Vot_haVotat NOT NULL
);

ALTER TABLE Vot ADD CONSTRAINT PK_Vot PRIMARY KEY (ciutada,votacio);


-------------------------------------------------------------------------------
-- Creaci taula CensCiutada
-------------------------------------------------------------------------------
CREATE TABLE CensCiutada(
  cens CONSTRAINT FK_cens REFERENCES Cens(codiCens),
  ciutada CONSTRAINT FK_CensCiutada REFERENCES Ciutada(codiAleatori)
);

ALTER TABLE CensCiutada ADD CONSTRAINT PK_cens PRIMARY KEY (cens,ciutada);


-------------------------------------------------------------------------------
-- Creaci taula CensVotacio
-------------------------------------------------------------------------------
CREATE TABLE CensVotacio(
  codiCens CONSTRAINT FK1_CensVotacio REFERENCES Cens(codiCens),
  votacio CONSTRAINT FK2_CensVotacio REFERENCES Votacio(codiVotacio)
);

ALTER TABLE CensVotacio ADD CONSTRAINT PK_CensVotacio PRIMARY KEY (codiCens,votacio);


-------------------------------------------------------------------------------
-- Creaci taula RegistreLog 
-------------------------------------------------------------------------------
CREATE TABLE RegistreLog(
  idLog NUMBER CONSTRAINT PK_idLog PRIMARY KEY,
  nomProcedEmmagatzemat VARCHAR2(100 CHAR) CONSTRAINT NN_log_nomProcedEmmagatzemat NOT NULL,
  parametresEntrada VARCHAR2(500 CHAR) CONSTRAINT NN_log_parametresEntrada NOT NULL,
  dataExecucio DATE CONSTRAINT NN_log_dataExecucio NOT NULL,
  RSP VARCHAR2(500 CHAR) CONSTRAINT NN_log_RSP NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Est1
-------------------------------------------------------------------------------
CREATE TABLE Est1(
  idEst1 NUMBER CONSTRAINT PK_idEst1 PRIMARY KEY,
  codiPais CONSTRAINT FK_Est1 REFERENCES Pais(codiPais)
		CONSTRAINT NN_Est1_codiPais NOT NULL,
  anyo NUMBER CONSTRAINT NN_Est1_any NOT NULL,
  nombreVotacFinalitz NUMBER CONSTRAINT NN_Ets1_nombreVotacFinalitz NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula Est2_3
-------------------------------------------------------------------------------
CREATE TABLE Est2_3(
  idEst2_3 NUMBER CONSTRAINT PK_idEst2_3 PRIMARY KEY,
  codiCens CONSTRAINT FK_Est2_3 REFERENCES Cens(codiCens)
		CONSTRAINT NN_Est2_3_codiCens NOT NULL,
  anyo NUMBER CONSTRAINT NN_Est2_3_any NOT NULL,
  valorMigPercenPartic FLOAT CONSTRAINT NN_Est2_3_valorMigPercenPartic NOT NULL,
  nombreVotacions NUMBER CONSTRAINT NN_Est2_3_nombreVotacions NOT NULL
);


-------------------------------------------------------------------------------
-- Creaci taula trEst
-------------------------------------------------------------------------------
CREATE TABLE trEst(
  idTrEst NUMBER CONSTRAINT PK_idTrEst PRIMARY KEY,
  codiLocalitat CONSTRAINT FK_TrEst REFERENCES Localitat(codiLocalitat)
		CONSTRAINT NN_TrEst_codiLocalitat NOT NULL,
  anyo NUMBER CONSTRAINT NN_TrEst_any NOT NULL,
  nombreVots NUMBER
);


-------------------------------------------------------------------------------
-- Creaci taula trEstVotCiu
-------------------------------------------------------------------------------
CREATE TABLE trEstVotCiu(
  ciutada CONSTRAINT FK_TrEstVotCiu REFERENCES Ciutada(codiAleatori),
  anyo NUMBER
);

ALTER TABLE trEstVotCiu ADD CONSTRAINT PK_trEstVotCiu PRIMARY KEY (ciutada,anyo);


-------------------------------------------------------------------------------
-- Creaci taula trEstNumVotacions
-------------------------------------------------------------------------------
CREATE TABLE trEstNumVotacions(
  ciutada CONSTRAINT FK_TrEstNumVotacions REFERENCES Ciutada(codiAleatori),
  votacions NUMBER
);

ALTER TABLE trEstNumVotacions ADD CONSTRAINT PK_trEstNumVotacions PRIMARY KEY (ciutada);


-------------------------------------------------------------------------------
-- Creacio taula Est4_5_6
-------------------------------------------------------------------------------
CREATE TABLE Est4_5_6(
  idEst4_5_6 NUMBER CONSTRAINT PK_idEst4_5_6 PRIMARY KEY,
  anyo NUMBER CONSTRAINT NN_Est4_5_6_any NOT NULL,
  votacioMesParticip VARCHAR2(30 CHAR) CONSTRAINT NN_Est4_5_6_votacioMesParticip NOT NULL,
  percVotMesParticip FLOAT CONSTRAINT NN_Est4_5_6_percVMParticip NOT NULL,
  localitMesVotsEmes VARCHAR2(50 CHAR) CONSTRAINT NN_Est4_5_6_localitMesVotsEmes NOT NULL,
  percenCiutMaiHaVot FLOAT CONSTRAINT NN_Est4_5_6_percenCiutMaiHaVot NOT NULL
);



-------------------------------------------------------------------------------
-- Creaci taula Est7_8_9_10
-------------------------------------------------------------------------------
CREATE TABLE Est7_8_9_10(
  idEst7_8_9_10 NUMBER CONSTRAINT PK_idEst7_8_9_10 PRIMARY KEY,
  votHistoricaMesParticipacio VARCHAR2(30 CHAR) CONSTRAINT NN_E7_vHistMesParticipacio NOT NULL,
  votMesDifOpcioMesMenysVotada VARCHAR2(30 CHAR) CONSTRAINT NN_E7_vMesDifOpMMVotada NOT NULL,
  votMenysDifOpcioMesMenysVotada VARCHAR2(30 CHAR) CONSTRAINT NN_E7_vMenysDifOpMMVotada NOT NULL,
  nombreMaximVotsUnCiutada NUMBER CONSTRAINT NN_E7_nombreMaximVotsUnCiutada NOT NULL,
  codiPaisCiutadaMaximVots VARCHAR2(2 CHAR) CONSTRAINT NN_E7_codiPaisCiutadaMaximVots NOT NULL,
  maxParticipacio FLOAT,
  maxDifVots FLOAT,
  minDifVots FLOAT
);


--*********************************************************************************************

-------------------------------------------------------------------------------
-- Definici del comptador de la taula trEst
-------------------------------------------------------------------------------
CREATE SEQUENCE seqIdTrEst INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouIdTrEst
  BEFORE INSERT ON TrEst
  FOR EACH ROW
BEGIN
  SELECT seqIdTrEst.NEXTVAL INTO :new.idTrEst
  FROM DUAL;
END nouIdTrEst;

-------------------------------------------------------------------------------
-- Definici del comptador de la taula Localitat
-------------------------------------------------------------------------------
CREATE SEQUENCE seqCodiLocalitat INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouCodiLocalitat
  BEFORE INSERT ON Localitat
  FOR EACH ROW
BEGIN
  SELECT seqCodiLocalitat.NEXTVAL INTO :new.codiLocalitat
  FROM DUAL;
END nouCodiLocalitat;

-------------------------------------------------------------------------------
-- Definici del comptador de la taula Regio
-------------------------------------------------------------------------------
CREATE SEQUENCE seqCodiRegio INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouCodiRegio
  BEFORE INSERT ON Regio
  FOR EACH ROW
BEGIN
  SELECT seqCodiRegio.NEXTVAL INTO :new.codiRegio
  FROM DUAL;
END nouCodiRegio;




-------------------------------------------------------------------------------
-- Definici del comptador de la taula Cens
-------------------------------------------------------------------------------
CREATE SEQUENCE seqCodiCens INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouCodiCens
  BEFORE INSERT ON Cens
  FOR EACH ROW
BEGIN
  SELECT seqCodiCens.NEXTVAL INTO :new.codiCens
  FROM DUAL;
END nouCodiCens;


-------------------------------------------------------------------------------
-- Creaci comptador per l'identificador de la Votaci
-------------------------------------------------------------------------------
CREATE SEQUENCE seqCodiVotacio INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouCodiVotacio
  BEFORE INSERT ON Votacio
  FOR EACH ROW
BEGIN
  SELECT seqCodiVotacio.NEXTVAL INTO :new.codiVotacio
  FROM DUAL;
END nouCodiVotacio;


-------------------------------------------------------------------------------
-- Definici del comptador de la taula Opcio
-------------------------------------------------------------------------------
CREATE SEQUENCE seqCodiOpcio INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouCodiOpcio
  BEFORE INSERT ON Opcio
  FOR EACH ROW
BEGIN
  SELECT seqCodiOpcio.NEXTVAL INTO :new.codiOpcio
  FROM DUAL;
END nouCodiOpcio;


-------------------------------------------------------------------------------
-- Definici del comptador de la taula registreLog
-------------------------------------------------------------------------------
CREATE SEQUENCE seqIdLog INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouIdLog
  BEFORE INSERT ON registreLog
  FOR EACH ROW
BEGIN
  SELECT seqIdLog.NEXTVAL INTO :new.idLog
  FROM DUAL;
END nouIdLog;


-------------------------------------------------------------------------------
-- Definici del comptador de la taula Est1
-------------------------------------------------------------------------------
CREATE SEQUENCE seqIdEst1 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouIdEst1
  BEFORE INSERT ON Est1
  FOR EACH ROW
BEGIN
  SELECT seqIdEst1.NEXTVAL INTO :new.idEst1
  FROM DUAL;
END nouIdEst1;


-------------------------------------------------------------------------------
-- Definici del comptador de la taula Est2_3
-------------------------------------------------------------------------------
CREATE SEQUENCE seqIdEst2_3 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouIdEst2_3
  BEFORE INSERT ON Est2_3
  FOR EACH ROW
BEGIN
  SELECT seqIdEst2_3.NEXTVAL INTO :new.idEst2_3
  FROM DUAL;
END nouIdEst2_3;


-------------------------------------------------------------------------------
-- Definici del comptador de la taula Est4_5_6
-------------------------------------------------------------------------------
CREATE SEQUENCE seqIdEst4_5_6 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouIdEst4_5_6
  BEFORE INSERT ON Est4_5_6
  FOR EACH ROW
BEGIN
  SELECT seqIdEst4_5_6.NEXTVAL INTO :new.idEst4_5_6
  FROM DUAL;
END nouIdEst4_5_6;


-------------------------------------------------------------------------------
-- Definici del comptador de la taula Est7_8_9_10
-------------------------------------------------------------------------------
CREATE SEQUENCE seqIdEst7_8_9_10 INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER nouIdEst7_8_9_10
  BEFORE INSERT ON Est7_8_9_10
  FOR EACH ROW
BEGIN
  SELECT seqIdEst7_8_9_10.NEXTVAL INTO :new.idEst7_8_9_10
  FROM DUAL;
END nouIdEst7_8_9_10;


-------------------------------------------------------------------------------
-- Definici dels sinonims per accedir-hi
-------------------------------------------------------------------------------
CREATE OR REPLACE PUBLIC SYNONYM TipusCens FOR TipusCens;
CREATE OR REPLACE PUBLIC SYNONYM EstatVotacio FOR EstatVotacio;
CREATE OR REPLACE PUBLIC SYNONYM Pais FOR Pais;
CREATE OR REPLACE PUBLIC SYNONYM Regio FOR Regio;
CREATE OR REPLACE PUBLIC SYNONYM Localitat FOR Localitat;
CREATE OR REPLACE PUBLIC SYNONYM Cens FOR Cens;
CREATE OR REPLACE PUBLIC SYNONYM Ciutada FOR Ciutada;
CREATE OR REPLACE PUBLIC SYNONYM Votacio FOR Votacio;
CREATE OR REPLACE PUBLIC SYNONYM Opcio FOR Opcio;
CREATE OR REPLACE PUBLIC SYNONYM Vot FOR Vot;
CREATE OR REPLACE PUBLIC SYNONYM CensCiutada FOR CensCiutada;
CREATE OR REPLACE PUBLIC SYNONYM CensVotacio FOR CensVotacio;
CREATE OR REPLACE PUBLIC SYNONYM RegistreLog FOR RegistreLog;
CREATE OR REPLACE PUBLIC SYNONYM Est1 FOR Est1;
CREATE OR REPLACE PUBLIC SYNONYM Est2_3 FOR Est2_3;
CREATE OR REPLACE PUBLIC SYNONYM Est4_5_6 FOR Est4_5_6;
CREATE OR REPLACE PUBLIC SYNONYM Est7_8_9_10 FOR Est7_8_9_10;
CREATE OR REPLACE PUBLIC SYNONYM TrEst FOR TrEst;
CREATE OR REPLACE PUBLIC SYNONYM TrEstNumVotacions FOR TrEstNumVotacions;
CREATE OR REPLACE PUBLIC SYNONYM TrEstVotCiu FOR TrEstVotCiu;
