ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula Cens
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaCens (
  p_descripcio IN Cens.descripcio%TYPE,
  p_pais IN Cens.pais%TYPE,
  p_regio IN Cens.regio%TYPE,
  p_localitat IN Cens.localitat%TYPE,
  p_tipusCens IN Cens.tipusCens%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  FaltenCamps EXCEPTION;

BEGIN

  parametresEntrada := 'p_descripcio: ' || p_descripcio || ', p_pais: ' || p_pais || 
    ', p_regio: ' || p_regio || ', p_localitat: ' || p_localitat || 
    ', p_tipusCens: ' || p_tipusCens;

  -- Comprovem depenent del tipus si t els camps informats
  -- Si s local calen tots els camps
  IF p_tipusCens='L' THEN
    RSP := '';
    IF p_pais IS NULL THEN
      RSP := 'ERROR: Per definir un cens local cal informar el pais';
    END IF;
    IF p_regio IS NULL THEN
      RSP := 'ERROR: Per definir un cens local cal informar la regi';
    END IF;
    IF p_localitat IS NULL THEN
      RSP := 'ERROR: Per definir un cens local cal informar la localitat';
    END IF;
    IF RSP != '' THEN
      RAISE FaltenCamps;
    END IF;
  END IF;

  -- Si s regional cal regi i pais
  IF p_tipusCens='R' THEN
    RSP := '';
    IF p_pais IS NULL THEN
      RSP := 'ERROR: Per definir un cens regional cal informar el pais';
    END IF;
    IF p_regio IS NULL THEN
      RSP := 'ERROR: Per definir un cens regional cal informar la regi';
    END IF;
    IF RSP != '' THEN
      RAISE FaltenCamps;
    END IF;
  END IF;

  -- Si s nacional cal el pais
  IF p_tipusCens='N' THEN
    RSP := '';
    IF p_pais IS NULL THEN
      RSP := 'ERROR: Per definir un cens nacional cal informar el pais';
      RAISE FaltenCamps;
    END IF;
  END IF;

  -- Inserim el nou registre
  INSERT INTO Cens (descripcio, pais, regio, localitat, tipusCens)
    VALUES (p_descripcio, p_pais, p_regio, p_localitat, p_tipusCens);

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta cens', parametresEntrada, RSP);
  
  RETURN RSP;

EXCEPTION
  -- Si falten camps
  WHEN FaltenCamps THEN
    gravarLog ('Alta cens', parametresEntrada, RSP);
    RETURN RSP;

  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1400 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta cens', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Cens
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION modificarCens (
  p_descripcio IN Cens.descripcio%TYPE,
  p_codiCens IN Cens.codiCens%TYPE,
  p_pais IN Cens.pais%TYPE,
  p_regio IN Cens.regio%TYPE,
  p_localitat IN Cens.localitat%TYPE,
  p_tipusCens IN Cens.tipusCens%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  FaltenCamps EXCEPTION;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_descripcio: ' || p_descripcio || ', p_pais: ' || p_pais || 
    ', p_regio: ' || p_regio || ', p_localitat: ' || p_localitat || 
    ', p_tipusCens: ' || p_tipusCens;

  -- Comprovem depenent del tipus si t els camps informats
  -- Si s local calen tots els camps
  IF p_tipusCens='L' THEN
    RSP := '';
    IF p_pais IS NULL THEN
      RSP := 'ERROR: Per definir un cens local cal informar el pais';
    END IF;
    IF p_regio IS NULL THEN
      RSP := 'ERROR: Per definir un cens local cal informar la regi';
    END IF;
    IF p_localitat IS NULL THEN
      RSP := 'ERROR: Per definir un cens local cal informar la localitat';
    END IF;
    IF RSP != '' THEN
      RAISE FaltenCamps;
    END IF;
  END IF;

  -- Si s regional cal regi i pais
  IF p_tipusCens='R' THEN
    RSP := '';
    IF p_pais IS NULL THEN
      RSP := 'ERROR: Per definir un cens regional cal informar el pais';
    END IF;
    IF p_regio IS NULL THEN
      RSP := 'ERROR: Per definir un cens regional cal informar la regi';
    END IF;
    IF RSP != '' THEN
      RAISE FaltenCamps;
    END IF;
  END IF;

  -- Si s nacional cal el pais
  IF p_tipusCens='N' THEN
    RSP := '';
    IF p_pais IS NULL THEN
      RSP := 'ERROR: Per definir un cens nacional cal informar el pais';
      RAISE FaltenCamps;
    END IF;
  END IF;

  -- Fem lupdate del registre
  UPDATE Cens SET descripcio=p_descripcio, pais=p_pais, regio=p_regio, 
      localitat=p_localitat, tipusCens=p_tipusCens
    WHERE codiCens = p_codiCens;

  -- Si dona error que no existeix el registre anem a l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b o s l'error controlat
  RSP := 'OK';
  gravarLog ('Modificaci cens', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el cens amb codi:' || p_codiCens;
    gravarLog ('Modificaci cens', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Modificaci cens', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Cens
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaCens (
  p_codiCens IN Cens.codiCens%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiCens: ' || p_codiCens;

  DELETE Cens
   WHERE codiCens = p_codiCens;

  -- Si dona error que no existeix anem a l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa cens', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el cens amb codi:' || p_codiCens;
    gravarLog ('Baixa cens', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa cens', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de getCodiCens
--  per obtenir el codi d'un cens amb la descripci del cens
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION getCodiCens (   
  p_descripcio IN Cens.descripcio%TYPE
)
RETURN number
IS
 codi number := 0;

BEGIN

  SELECT codiCens INTO codi
  FROM Cens
  WHERE descripcio = p_descripcio;

  RETURN codi;

EXCEPTION
  WHEN OTHERS THEN
    RETURN codi;  

END;
