ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;


-------------------------------------------------------------------------------
-- Definici de lalta a la taula CensCiutada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaCensCiutada (
  p_cens IN Cens.codiCens%TYPE,
  p_ciutada IN Ciutada.codiAleatori%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := 'p_cens: ' || p_cens || ', p_ciutada: ' || p_ciutada;

  -- Inserim el nou registre
  INSERT INTO CensCiutada (cens, ciutada)
    VALUES (p_cens, p_ciutada);

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta cens ciutad', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1400 THEN
      RSP := 'ERROR: El cens i el ciutad sn camps obligatoris';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta cens ciutad', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula CensCiutada
-- en aquest cas no hi ha modificaci doncs els dos camps formen l'index
-- Per a fer canvis cal esborrar el registre i fer-lo de nou.
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula CensCiutada
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaCensCiutada (
  p_cens IN Cens.codiCens%TYPE,
  p_ciutada IN Ciutada.codiAleatori%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_cens: ' || p_cens || ', p_ciutada: ' || p_ciutada;

  -- Esborrem el registre
  DELETE CensCiutada
   WHERE cens = p_cens AND ciutada = p_ciutada;

  -- Si dona error que no existeix anem a l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa cens ciutad', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el cens amb codi: ' || p_cens || ' o el ciutada amb codi: ' || p_ciutada;
    gravarLog ('Baixa cens ciutad', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa cens ciutad', parametresEntrada, RSP);
    RETURN RSP;

END;