ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula Localitat
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaLocalitat (
  p_nomLocalitat IN Localitat.nomLocalitat%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := 'p_nomLocalitat: ' || p_nomLocalitat;

  -- Inserim el nou registre
  INSERT INTO Localitat (nomLocalitat)
    VALUES (p_nomLocalitat);

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta localitat', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1 THEN
      RSP := 'ERROR: La localitat ' || p_nomLocalitat || ' ja existeix a la base de dades';
    ELSIF SQLCODE = -1400 THEN
      RSP := 'ERROR: El nom de la localitat s un camp obligatori';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta localitat', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Localitat
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION modificarLocalitat (
  p_codiLocalitat IN Localitat.codiLocalitat%TYPE,
  p_nomLocalitat IN Localitat.nomLocalitat%TYPE
)  RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiLocalitat: ' || p_codiLocalitat || ', p_nomLocalitat:' || p_nomLocalitat;

  -- Actualitzem els valors del registre
  UPDATE Localitat SET nomLocalitat = p_nomLocalitat
    WHERE codiLocalitat = p_codiLocalitat;

  -- Si no existeix llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b o s l'error controlat
  RSP := 'OK';
  gravarLog ('Modificaci localitat', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix la localitat amb codi:' || p_codiLocalitat;
    gravarLog ('Modificaci localitat', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: El nom de la localitat s un camp obligatori';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Modificaci localitat', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Localitat
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaLocalitat (
  p_codiLocalitat IN Localitat.codiLocalitat%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiLocalitat: ' || p_codiLocalitat;

  -- Esborrem el registre
  DELETE Localitat
   WHERE codiLocalitat = p_codiLocalitat;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa localitat', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix la localitat amb codi:' || p_codiLocalitat;
    gravarLog ('Baixa localitat', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa localitat', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de getCodiLocalitat
--  per obtenir el codi d'una localitat amb el nom de la localitat
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION getCodiLocalitat (   
  p_nom IN Localitat.nomLocalitat%TYPE
)
RETURN number
IS
 codi number := 0;

BEGIN

  SELECT codiLocalitat INTO codi
  FROM Localitat
  WHERE nomLocalitat = p_nom;

  RETURN codi;

EXCEPTION
  WHEN OTHERS THEN
    RETURN codi;  
END;
