ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula Regio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaRegio (
  p_nomRegio IN Regio.nomRegio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := 'p_nomRegio: ' || p_nomRegio;

  -- Inserim el nou registre
  INSERT INTO Regio (nomRegio)
    VALUES (p_nomRegio);

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta regi', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
     IF SQLCODE = -1 THEN
       RSP := 'ERROR: La regi ' || p_nomRegio || ' ja existeix a la base de dades';
     ELSIF SQLCODE = -1400 THEN
       RSP := 'ERROR: El nom de la regi s un camp obligatori';
     END IF;

     -- Si dona un error diferent d'aquests el mostrem tal qual
     IF RSP IS NULL THEN
       RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
     END IF;
     gravarLog ('Alta regi', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Regio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION modificarRegio (
  p_codiRegio IN Regio.codiRegio%TYPE,
  p_nomRegio IN	Regio.nomRegio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiRegio: ' || p_codiRegio || ', p_nomRegio:' || p_nomRegio;

  -- Actualitzem els valors del registre
  UPDATE Regio SET nomRegio = p_nomRegio
    WHERE codiRegio = p_codiRegio;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b o s l'error controlat
  RSP := 'OK';
  gravarLog ('Modificaci regi', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix la regi amb codi:' || p_codiRegio;
    gravarLog ('Modificaci regi', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: El nom de la regi s un camp obligatori';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Modificaci regi', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Regio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaRegio (
  p_codiRegio IN Regio.codiRegio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiRegio: ' || p_codiRegio;

  -- Esborrem el registre
  DELETE Regio
   WHERE codiRegio = p_codiRegio;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa regi', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix la regi amb codi:' || p_codiRegio;
    gravarLog ('Baixa regi', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa regi', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de getCodiRegio
--  per obtenir el codi d'una regi amb el nom de la regi
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION getCodiRegio (   
  p_nom IN Regio.nomRegio%TYPE
)
RETURN number
IS
 codi number := 0;

BEGIN

  SELECT codiRegio INTO codi
  FROM Regio
  WHERE nomRegio = p_nom;

  RETURN codi;

EXCEPTION
  WHEN OTHERS THEN
    RETURN codi;  
END;