ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;


-------------------------------------------------------------------------------
-- Definici de lalta a la taula Pais
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaPais (
  p_codiPais IN Pais.codiPais%TYPE,
  p_nomPais IN Pais.nomPais%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := 'p_codiPais: ' || p_codiPais || ', p_nomPais:' || p_nomPais;

  -- Inserim el nou registre
  INSERT INTO Pais (codiPais,nomPais)
    VALUES (p_codiPais, p_nomPais);

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta pais', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1 THEN
      RSP := 'ERROR: El pais ' || p_nomPais || ' ja existeix a la base de dades';
    ELSIF SQLCODE = -1400 THEN
      RSP := 'ERROR: El nom del pais s un camp obligatori';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta pais', parametresEntrada, RSP);
    RETURN RSP;
END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Pais
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION modificarPais (
  p_codiPais IN Pais.codiPais%TYPE,
  p_nomPais IN Pais.nomPais%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiPais: ' || p_codiPais || ', p_nomPais:' || p_nomPais;

  -- Actualitzem el registre
  UPDATE Pais SET nomPais = p_nomPais
    WHERE codiPais = p_codiPais;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b o s l'error controlat
  RSP := 'OK';
  gravarLog ('Modificaci pais', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el pais amb codi:' || p_codiPais;
    gravarLog ('Modificaci pais', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: El nom del pais s un camp obligatori';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Modificaci pais', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Pais
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaPais (
  p_codiPais IN Pais.codiPais%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  EsAEstadistica EXCEPTION;
  NoExisteix EXCEPTION;
  HiHa number;

BEGIN

  parametresEntrada := 'p_codiPais: ' || p_codiPais;

  -- Mirem si est a alguna estadstica
  SELECT COUNT(*) INTO HiHa
   FROM Est1
   WHERE codiPais = p_codiPais;

  -- Si hi s llancem excepci
  IF HiHa>0 THEN
    RAISE EsAEstadistica;
  END IF;

  -- Esborrar registre
  DELETE Pais
   WHERE codiPais = p_codiPais;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa pais', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN EsAEstadistica THEN
    RSP := 'ERROR: No es pot esborrar el pais amb codi:' || p_codiPais || ' doncs hi ha estadstiques a Est1.';
    gravarLog ('Baixa pais', parametresEntrada, RSP);
    RETURN RSP;

  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix el pais amb codi:' || p_codiPais;
    gravarLog ('Baixa pais', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa pais', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici del trigger a lalta de la taula Pais
-- Cal actualitzar l'estadstica Est1 que va per pasos
-------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER insert_Pais
  AFTER INSERT ON Pais
  FOR EACH ROW
BEGIN
  INSERT INTO Est1 ( codiPais, anyo, nombreVotacFinalitz)
   VALUES ( :new.codiPais, to_char (SYSDATE, 'yyyy'), 0);
END insert_Pais;


-------------------------------------------------------------------------------
-- Definici del trigger a la baixa de la taula Pais
-- Cal actualitzar l'estadstica Est1 que va per pasos
-------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER delete_Pais
  BEFORE DELETE ON Pais
  FOR EACH ROW
BEGIN
  DELETE FROM Est1
   WHERE codiPais = :old.codiPais;
END delete_Pais;


-------------------------------------------------------------------------------
-- Definici de getCodiPais
--  per obtenir el codi d'un pas amb el nom del pas
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION getCodiPais (   
  p_nom IN Pais.nomPais%TYPE
)
RETURN number
IS
 codi Varchar2(2) := '';

BEGIN

  SELECT codiPais INTO codi
  FROM Pais
  WHERE nomPais = p_nom;

  RETURN codi;

EXCEPTION
  WHEN OTHERS THEN
    RETURN codi;  
END;