ALTER SESSION SET CURRENT_SCHEMA = VOTACIONS;
COMMIT;

-------------------------------------------------------------------------------
-- Definici de lalta a la taula Opcio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION altaOpcio (
  p_descripcioOpcio IN Opcio.descripcioOpcio%TYPE,
  p_votacio IN Votacio.codiVotacio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;

BEGIN

  parametresEntrada := 'p_descripcioOpcio: ' || p_descripcioOpcio || ', p_votacio: ' || P_Votacio;

  -- Inserim el nou registre
  INSERT INTO Opcio (votacio, nombreVots, percentatge, descripcioOpcio)
    VALUES (p_votacio, 0, 0, p_descripcioOpcio);

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Alta opci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  -- En cas d'error mirem els ms comuns
  WHEN OTHERS THEN
    IF SQLCODE = -1400 THEN
      RSP := 'ERROR: La descripci de lopci s un camp obligatori';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Alta opci', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la modificaci de la taula Opcio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION modificarOpcio (
  p_codiOpcio IN Opcio.codiOpcio%TYPE,
  p_descripcioOpcio IN Opcio.descripcioOpcio%TYPE,
  p_votacio IN Votacio.codiVotacio%TYPE,
  p_nombreVots IN Opcio.nombreVots%TYPE,
  p_percentatge IN Opcio.percentatge%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiOpcio: ' || p_codiOpcio || 'p_descripcioOpcio: ' || p_descripcioOpcio || 
     ', p_votacio: ' || p_Votacio || ', p_nombreVots: ' || p_nombreVots || 
     ', p_percentatge: ' || p_percentatge;

  -- Actualitzem el registre mirant si els valors p_nombreVots i p_percentatge sn nulls
  --  i en aquest cas no els actualitzem
  IF p_nombreVots IS NULL AND p_percentatge IS NULL THEN
    UPDATE Opcio SET codiOpcio=p_codiOpcio, descripcioOpcio=p_descripcioOpcio, votacio=p_Votacio
      WHERE codiOpcio = p_codiOpcio;

  ELSE
    UPDATE Opcio SET codiOpcio=p_codiOpcio, descripcioOpcio=p_descripcioOpcio, votacio=p_Votacio, 
        nombreVots=p_nombreVots, percentatge=p_percentatge
      WHERE codiOpcio = p_codiOpcio;
  END IF;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Modificaci opci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix lopci amb codi:' || p_codiOpcio;
    gravarLog ('Modificaci opci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    IF SQLCODE = -1407 THEN
      RSP := 'ERROR: Hi ha camps obligatoris no informats';
    END IF;

    -- Si dona un error diferent d'aquests el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Modificaci opci', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de la baixa de la taula Opcio
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION baixaOpcio (
  p_codiOpcio IN Opcio.codiOpcio%TYPE
) RETURN RegistreLog.RSP%TYPE
IS
  parametresEntrada RegistreLog.parametresEntrada%TYPE;
  RSP RegistreLog.RSP%TYPE;
  NoExisteix EXCEPTION;

BEGIN

  parametresEntrada := 'p_codiOpcio: ' || p_codiOpcio;

  -- Esborrem el registre
  DELETE Opcio
   WHERE codiOpcio = p_codiOpcio;

  -- Si no existeix el registre llancem l'excepci
  IF SQL%NOTFOUND THEN
    RAISE NoExisteix;
  END IF;

  -- Fem el registre del log si tot ha anat b
  RSP := 'OK';
  gravarLog ('Baixa Opci', parametresEntrada, RSP);

  RETURN RSP;

EXCEPTION
  WHEN NoExisteix THEN
    RSP := 'ERROR: No existeix lopci amb codi:' || p_codiOpcio;
    gravarLog ('Baixa opci', parametresEntrada, RSP);
    RETURN RSP;

  WHEN OTHERS THEN
    -- Si dona un error diferent el mostrem tal qual
    IF RSP IS NULL THEN
      RSP := 'ERROR: ' || SQLCODE || SUBSTR (SQLERRM, 1, 480);
    END IF;
    gravarLog ('Baixa opci', parametresEntrada, RSP);
    RETURN RSP;

END;


-------------------------------------------------------------------------------
-- Definici de getCodiOpcio
--  per obtenir el codi d'una opci amb la votaci i la descripci de l'opci
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION getCodiOpcio (   
  p_votacio IN Votacio.codiVotacio%TYPE,
  p_descOpcio IN Opcio.descripcioOpcio%TYPE
) RETURN number
IS
 codi number := 0;

BEGIN

  SELECT codiOpcio INTO codi
  FROM Opcio
  WHERE votacio = p_votacio AND descripcioOpcio = p_descOpcio;

  RETURN codi;

EXCEPTION
  WHEN OTHERS THEN
    RETURN codi;  
END;